# -*- coding: UTF-8 -*-
import tensorflow as tf
from lib.networks.network import Network


class Resnet34_lstm(Network):
    def __init__(self, embedding_size, trainable=True):
        self.L = 11
        self.inputs = []
        self.feature = tf.placeholder(tf.float32, shape=[None, self.L, embedding_size],
                                      name='feature')
        self.label = tf.placeholder(tf.float32, shape=[None, 1], name='label')
        self.mask = tf.placeholder(tf.float32, shape=[None, 1], name='mask')

        self.layers = dict({'feature': self.feature, 'label': self.label,
                            'L': self.L, 'mask': self.mask, })
        self.trainable = trainable
        self.setup()

    def setup(self):
        (self.feed('feature')
         .conv1d(3, 8, 1, biased=True, relu=False, name='conv1')
         .batch_normalization(relu=False, name='bn_conv1', is_training=self.trainable)
         .relu(name='conv1_relu')

         .conv1d(3, 8, 1, biased=True, relu=False, name='conv2')
         .batch_normalization(relu=False, name='bn_conv2', is_training=self.trainable)
         .relu(name='conv2_relu')

         .conv1d(3, 8, 1, biased=True, relu=False, name='conv3')
         .batch_normalization(relu=False, name='bn_conv3', is_training=self.trainable))

        (self.feed('conv1_relu',
                   'bn_conv3')
         .add(name='add1')
         .relu(name='add1_relu')

         .conv1d(3, 2, 1, biased=True, relu=False, name='conv4')
         .fc(self.L*2, 1, relu=False, name='label_fullconn')
         .sigmoid(name='label_logits'))
