# -*- coding: UTF-8 -*-
'''functions used in SPIDER3.
'''
import numpy as np
import pandas as pd


def spd3_feature_sincos(x, seq):
    ASA = x[:, 0]
    rnam1_std = "ACDEFGHIKLMNPQRSTVWY-"
    ASA_std = (115, 135, 150, 190, 210, 75, 195, 175, 200, 170,
               185, 160, 145, 180, 225, 115, 140, 155, 255, 230, 1)
    dict_rnam1_ASA = dict(zip(rnam1_std, ASA_std))
    ASA_div = np.array([dict_rnam1_ASA[i] for i in seq])
    ASA = (ASA / ASA_div)[:, None]
    angles = x[:, 1:5]
    HSEa = x[:, 5:7]
    HCEprob = x[:, 7:10]
    angles = np.deg2rad(angles)
    angles = np.concatenate([np.sin(angles), np.cos(angles)], 1)
    return ASA, angles, HSEa, HCEprob


def read_spd33(fname, seq):
    with open(fname, 'r') as f:
        spd3_features = pd.read_csv(f, delim_whitespace=True).values[:, 3:].astype(float)
    tmp_ASA, tmp_angles, tmp_HSEa, tmp_HCEprob = spd3_feature_sincos(spd3_features, seq)
    if (tmp_ASA.shape[0] != len(seq)
            or tmp_angles.shape[0] != len(seq)
            or tmp_HSEa.shape[0] != len(seq)
            or tmp_HCEprob.shape[0] != len(seq)):
        raise ValueError('HHM file is in wrong format or incorrect!')
    return tmp_ASA, tmp_angles, tmp_HSEa, tmp_HCEprob


def read_pssm(fname, seq):
    num_pssm_cols = 44
    pssm_col_names = [str(j) for j in range(num_pssm_cols)]
    with open(fname, 'r') as f:
        tmp_pssm = pd.read_csv(f, delim_whitespace=True,
                               names=pssm_col_names).dropna().values[:, 2:22].astype(float)
    if tmp_pssm.shape[0] != len(seq):
        raise ValueError('PSSM file is in wrong format or incorrect!')
    return tmp_pssm


def read_hhm(fname, seq):
    num_hhm_cols = 22
    hhm_col_names = [str(j) for j in range(num_hhm_cols)]
    with open(fname, 'r') as f:
        hhm = pd.read_csv(f, delim_whitespace=True, names=hhm_col_names)
    pos1 = (hhm['0'] == 'HMM').idxmax() + 3
    hhm = hhm[pos1:-1].values[:, :num_hhm_cols].reshape([-1, 44])
    hhm[hhm == '*'] = '9999'
    if hhm.shape[0] != len(seq):
        raise ValueError('HHM file is in wrong format or incorrect!')
    return hhm[:, 2:-12].astype(float)
