# -*- coding: UTF-8 -*-
import os
import os.path as osp
import tensorflow as tf
import numpy as np
import argparse
import heapq

from lib.utils.sampleop import to_block
from lib.networks.Resnet34_lstm import Resnet34_lstm
from lib.utils.feature_combine import feature_combine

CONFIG = tf.ConfigProto(allow_soft_placement=True)
CONFIG.gpu_options.allocator_type = 'BFC'

modifier = {'A': 0.62, 'C': 0.29, 'D': -0.9, 'E': -0.74, 'F': 1.19, 'G': 0.48,
            'H': -0.4, 'I': 1.38, 'K': -1.5, 'L': 1.06, 'M': 0.64, 'N': -0.78,
            'P': 0.12, 'Q': -0.85, 'R': -2.53, 'S': -0.18, 'T': -0.05, 'V': 1.08,
            'W': 0.81, 'Y': 0.26}


os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('prot_path', type=str, help='prot_path')
    args = parser.parse_args()
    prot_path = args.prot_path

    cur_path = os.path.split(os.path.realpath(__file__))[0]
    MODEL_DIR = cur_path + '/models'
    HMM_DIR = prot_path + '/features/HMM'
    SPD33_DIR = prot_path + '/features/spd33'
    TEST_DIR = prot_path + '/fasta'
    TEST_PRED_DIR = prot_path + '/prediction'
    TEST_LIST = os.listdir(TEST_DIR)
    TEST_LIST = list(filter(lambda in_str: in_str.endswith('.fasta'), TEST_LIST))
    TEST_NUM = len(TEST_LIST)

    WINSIZE = 11
    HALF_LEN = int((WINSIZE-1) / 2)

    whole_feature = feature_combine(TEST_DIR, TEST_LIST, SPD33_DIR, HMM_DIR)

    with tf.variable_scope("res_lstm_model", reuse=None):
        eval_network = Resnet34_lstm(34, False)
    eval_sig = eval_network.get_output('label_logits')

    MODELS = os.listdir(MODEL_DIR)
    for imodel in MODELS:
        imodel_path = MODEL_DIR + '/' + imodel
        mean = np.load(imodel_path + '/mean.npy')
        std = np.load(imodel_path + '/std.npy')

        saver = tf.train.Saver()
        with tf.Session(config=CONFIG) as sess:
            saver.restore(sess, imodel_path + '/model')
            for ith, ifeature in enumerate(whole_feature):
                name = TEST_LIST[ith]
                name = name[:name.find('.')]
                with open(osp.join(TEST_DIR, TEST_LIST[ith]), 'r') as f:
                    seq = f.readlines()[-1]
                feature = (ifeature - mean) / std
                hydro = np.array([])
                for j in range(len(seq)):
                    hydro = np.append(hydro, modifier[seq[j]])
                hydro = hydro[..., np.newaxis]
                feature = np.concatenate([feature, hydro], 1)

                batch_feature_tmp = np.zeros((feature.shape[0], WINSIZE, feature.shape[1]))
                feature_pad = np.zeros([WINSIZE, feature.shape[1]])
                for j in range(feature.shape[0]):
                    feature_pad[int(HALF_LEN - (j - max(j - HALF_LEN, 0))):
                                int(HALF_LEN + (min(j + HALF_LEN + 1, feature.shape[0]) - j)), :] = \
                        feature[int(max(j - HALF_LEN, 0)):
                                int(min(j + HALF_LEN + 1, feature.shape[0])), :]
                    batch_feature_tmp[j, ...] = feature_pad[np.newaxis, ...]
                    feature_pad = np.zeros([WINSIZE, feature.shape[1]])

                feed_dict = {eval_network.feature: batch_feature_tmp, }
                [prediction] = sess.run(fetches=[eval_sig], feed_dict=feed_dict)
                prediction = np.squeeze(prediction)
                np.savetxt(TEST_PRED_DIR + '/' + name + '.' + imodel, prediction)

    for ifile in TEST_LIST:
        name = ifile[:ifile.find('.')]
        with open(osp.join(TEST_DIR, ifile), 'r') as f:
            seq = f.readlines()[-1]
        pred = np.zeros(len(seq))

        results = os.listdir(TEST_PRED_DIR)
        results = list(filter(lambda in_str: in_str.startswith(name), results))

        for i_result in results:
            sin_pred = np.loadtxt(TEST_PRED_DIR + '/' + i_result)
            pred += sin_pred

        pred /= len(results)
        np.savetxt(TEST_PRED_DIR + '/' + name + '.pred', pred)

        pred_blocks = to_block(np.where(pred >= 0.4)[0])
        del_ind = np.array([])
        for ind in range(pred_blocks.shape[0]):
            if pred_blocks[ind, 1] - pred_blocks[ind, 0] + 1 < 6:
                del_ind = np.append(del_ind, ind)
        pred_blocks = np.delete(pred_blocks, del_ind, 0)

        pred_bin = np.zeros(len(seq))
        if pred_blocks.shape[0] != 0:
            max_pred = np.zeros(pred_blocks.shape[0])
            for i in range(pred_blocks.shape[0]):
                max_pred[i] = max(pred[pred_blocks[i, 0] : pred_blocks[i, 1]+1])

            ness = np.where(max_pred > 0.65)[0]
            if ness.size > 1:
                select = heapq.nlargest(2, range(len(max_pred)), max_pred.take)
                pred_blocks = pred_blocks[select, :]
            else:
                select = np.argmax(max_pred)
                pred_blocks = pred_blocks[select, :]
                pred_blocks = pred_blocks[np.newaxis, ...]

            for ind in range(pred_blocks.shape[0]):
                pred_bin[pred_blocks[ind, 0] : pred_blocks[ind, 1]+1] = 1

        np.savetxt(TEST_PRED_DIR + '/' + name + '.pred_bin', pred_bin)

    all_files = os.listdir(TEST_PRED_DIR)
    preds = list(filter(lambda in_str: in_str.endswith('pred'), all_files))
    bins = list(filter(lambda in_str: in_str.endswith('bin'), all_files))
    results = preds + bins
    rm_files = list(set(all_files) - set(results))

    for file in rm_files:
        os.system('rm ' + TEST_PRED_DIR + '/' + file)
